﻿using System;
using System.Collections.Generic;

namespace XNAWinformReversi.GameLogic
{
    public class BoardModel
    {
        enum Piece { Empty , White , Black }; 
        public int[,] _board2DArray { get; set; }
        private List<Direction> _directionList;
        private int _currentPlayer;
        public List<Square> legalMovesList { get; set; }
        public List<Square> piecesToFlipList { get; set; }
        private int _blackPieceCount = 0;
        private int _whitePieceCount = 0;

        public BoardModel()
        {
            _board2DArray = new int[8, 8];
            resetBoardArray();
            loadDirectionList();
            legalMovesList = new List<Square>();
            piecesToFlipList = new List<Square>();
        }

        public int getSquareValue(int x, int y)
        { 
            // check to see if x,y is on the board if so return the (x, y)square value.
            int value = -1;
            if(isOnBoard(x, y))
            {
                value = _board2DArray[x, y];
            }
            return value;
        }

        public void setSquareValue(int color, int x, int y)
        {
            // if passed a valid color and (x, y) position set the (x, y) square value.
            if(isOnBoard(x, y))
            {
                if (color == (int)Piece.Empty || color == (int)Piece.Black || color == (int)Piece.White)
                {
                    _board2DArray[x, y] = color;
                }
            }
            
        }

        public int getCurrentPlayer()
        {
            return _currentPlayer;
        }

        public void setCurrentPlayer(int color)
        {
            // set the current player if passed a valid color.
            if (color == (int)Piece.Black || color == (int)Piece.White)
            {
                _currentPlayer = color;
            }
        }

        public int getWhitePieceCount()
        {
            return _whitePieceCount;
        }

        public int getBlackPieceCount()
        {
            return _blackPieceCount;
        }

        public bool loadLegalMoves(int color)
        {
            // mehtod will load the legalMovesList with legal move Square objects.
            // method will also return true if there are legal moves for color, false if not.

            legalMovesList.Clear();
            bool legalMoveExists = false;

            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    if (loadPiecesToFlip(color, x, y))
                    {
                        legalMoveExists = true;
                        Square temp = new Square(x, y);
                        legalMovesList.Add(temp);
                    }
                }
            }
            return legalMoveExists;
        }

        public bool loadPiecesToFlip(int color, int xStart, int yStart)
        {
            // mehtod loads piecesToFlipList and returns true if there is a legal move 
            // from xStart, yStart for given color, false if no valid move.

            bool isLegalMove = false;
            piecesToFlipList.Clear();
            int opponent = getOpponentColor(color);

            // Return false if passed x,y is already occupied or not on the board.
            if (!isOnBoard(xStart, yStart) || getSquareValue(xStart, yStart) > 0) 
            { 
                return false; 
            }

            // look in all 8 directions for legal moves. if found add them too piecesToFlipList.
            foreach (Direction i in _directionList)
            {
                int x = xStart;
                int y = yStart;
                // step in current direction.
                x += i.xDir;
                y += i.yDir;

                // if opponent piece is found keep stepping in current direction.
                while (getSquareValue(x, y) == opponent)
                {
                    x += i.xDir;
                    y += i.yDir;
                    if (!isOnBoard(x, y)) { break; }
                }
                // if we found a legal move step backward until start position, adding to piecesToFlipList each step.
                if (getSquareValue(x, y) == color)
                {
                    while (true)
                    {
                        x -= i.xDir;
                        y -= i.yDir;
                        if (x == xStart && y == yStart) { break; }
                        Square temp = new Square(x, y);
                        piecesToFlipList.Add(temp);
                        isLegalMove = true;
                    }
                }

            }
            return isLegalMove;
        }

        public void newGame()
        {
            // setup board2DArray and model for new game.
            // set currentPlayer to Black
            // load legalMoves
            resetBoardArray();
            _currentPlayer = (int)Piece.Black;
            setSquareValue((int)Piece.White, 3, 3);
            setSquareValue((int)Piece.Black, 4, 3);
            setSquareValue((int)Piece.Black, 3, 4);
            setSquareValue((int)Piece.White, 4, 4);
            loadLegalMoves(_currentPlayer);
            loadPieceCount();
        }

        private void resetBoardArray()
        {
            // sets all elements in board2DArray to 0(empty).
            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    setSquareValue(0, x, y);
                }
            }
        }

        private void loadDirectionList()
        {
            // loads direction list which will be used to look in all 8 directions for valid moves.
            _directionList = new List<Direction>();
            Direction north = new Direction(0, -1);
            Direction northEast = new Direction(1, -1);
            Direction east = new Direction(1, 0);
            Direction southEast = new Direction(1, 1);
            Direction south = new Direction(0, 1);
            Direction southWest = new Direction(-1, 1);
            Direction west = new Direction(-1, 0);
            Direction northWest = new Direction(-1, -1);
            _directionList.Add(north);
            _directionList.Add(northEast);
            _directionList.Add(east);
            _directionList.Add(southEast);
            _directionList.Add(south);
            _directionList.Add(southWest);
            _directionList.Add(west);
            _directionList.Add(northWest);
        }

        public void loadPieceCount()
        {
            // loads _blackPieceCount and _whitePieceCount
            _blackPieceCount = 0;
            _whitePieceCount = 0;

            for (int x = 0; x < 8; x++)
            {
                for (int y = 0; y < 8; y++)
                {
                    int temp = getSquareValue(x, y);
                    if (temp == (int)Piece.Black) 
                    { 
                        _blackPieceCount++; 
                    }
                    else if (temp == (int)Piece.White) 
                    { 
                        _whitePieceCount++; 
                    }
                }
            }
        }

        public int getOpponentColor(int color)
        {
            // returns opponent color, returns -1 if passed a non valid color.
            int opponent = -1;
            if (color == (int)Piece.Black) 
            { 
                opponent = (int)Piece.White; 
            }
            else 
            { 
                opponent = (int)Piece.Black; 
            }
            return opponent;
        }

        public int getWinner()
        {
            // pre-condition assumes controller has determined gameOver
            // returns winner, 0 if tie.
            int winner = 0;
            if (getBlackPieceCount() < getWhitePieceCount())
            {
                winner = (int)Piece.White;
            }
            else if (getWhitePieceCount() < getBlackPieceCount())
            {
                winner = (int)Piece.Black;
            }
            return winner;
        }

        private bool isOnBoard(int x, int y)
        {
            // returns true if x,y coordinate is on the board.
            return (x >= 0 && x <= 7 && y >= 0 && y <= 7);
        }

    }
}
